@@ -1,6 +1,6 @@
 /*
  * PL/scheme, Procedural Language Handler
- * $Id: plscheme.c,v 1.24 2006/12/09 19:31:04 knt Exp $
+ * $Id: plscheme.c,v 1.25 2008/02/24 09:38:24 knt Exp $
  *
  * Copyright (c) 2006, Volkan YAZICI <yazicivo@ttnet.net.tr>
  * All rights reserved.
@@ -3345,6 +3345,27 @@ spi_prepare(SCM in_command, SCM in_argtypes)
 				  scm_list_3(scm_from_locale_string(FUNC_NAME),
 					  		 in_command, in_argtypes));
 
+	/*
+	 * Move plan into a permanent memory context. Guile GC will handle
+	 * it if needed. (Also, by this way, we avoid freeing a plan.
+	 * Right now, plan is in the SPI procCxt, which will go away at
+	 * function end.)
+	 */
+	GUILE_TRY();
+	{
+		void	*tmpret;
+
+		tmpret = ret;
+		ret = SPI_saveplan(tmpret);
+		SPI_freeplan(tmpret);
+	}
+	GUILE_END_TRY();
+
+	if (!ret)
+		scm_throw(convert_from_spi(SPI_result),
+				  scm_list_3(scm_from_locale_string(FUNC_NAME),
+							 in_command, in_argtypes));
+
 	/*
 	 * Initialize SMOB to return to user.
 	 */
